br = read.table("br.csv", sep = ",", header = T) # reads the data from a csv file                               
attach(br) 


## 1. ################################################
## Principal component analysis

# preparation of input data and output: standardization
y = scale(price)
X = scale(data.frame(sqft, sqft^2, sqft^3, Age,
                     Age^2, Age^3, sqft*Age, Baths, Bedrooms));

## Eigendecomposition of the correlation matrix
S = cor(X)
print(S, digits = 2)
print(eigen(S), digits = 2)

## Obtain the principal components directly
pc.br = princomp(X);
Z = pc.br$scores; # principal components scores

pairs(Z[,1:4]);   # scatterplot of the first 4 pc's
cov(Z[,1:4])
plot(pc.br);      # bar plot of the eigenvalues (screeplot)
pc.br$sdev        # Standard deviation of the pc's 
par(mfrow = c(1, 1))

## Verifying outcomes of different procedures
eigen(S)$values
diag(var(Z))  # pc's are orthogonal and have decresing variances
# It holds up to a sign (not really important)
round((X %*% eigen(S)$vectors)[,2], 2) == round(Z[,2], 2)

## BIPLOT
# It summarises information about the first two principal components of our
# data in terms of: principal component scores (primary axes),
# and loadings of the variables on the principal components (secondary axes).
biplot(princomp(X), cex = 0.6, col = c(2,4))
# retrieving values on the secondary axes
pc.br$loadings[, 1] / pc.br$sdev[1] * 100
pc.br$loadings[, 2] / pc.br$sdev[2] * 100


## 2. ################################################
## Principal component regression

p = ncol(Z)
N = nrow(Z)
AIC = rep("NaN", p)
for (i in 1 : p) 
{
  pcr = lm(y ~ - 1 + Z[, 1 : i]) # we do not use the intercept because variables were scaled
  
  print(summary(pcr))
  AIC[i] = 2 * log(summary(pcr)$sigma) + 2 * i / N
}

plot(1 : p, AIC)

# would you support a strategy that includes only the first
# m pc's?
df = data.frame(y, Z)
Ntot = nrow(br)  # total n. of obs.
set.seed(123)    # random seed 
N = 600          # size of training sample
s = sample(1:Ntot, N)  # index of selected units
train = df[s, ]   # training sample
test = df[-s, ]   # validation sample

pc.regr.full = lm(y ~ - 1 + ., data = train)
pc.regr.short = lm(y ~ - 1 + Comp.1 + Comp.2 + Comp.3 + Comp.4 +
                     Comp.5 + Comp.6, data = train)


pred.full = predict(pc.regr.full, newdata =  test)
pred.short = predict(pc.regr.short, newdata =  test)
mse.full = mean((test$y - pred.full)^2)
mse.short = mean((test$y - pred.short)^2)
100 * (1- mse.short/mse.full)  # percent gain in accuracy due to model selection

